import os
import sys
from argparse import ArgumentParser
from hadmlservices import service


def make_parser():
    parser = ArgumentParser()
    parser.add_argument(
        'root_dir',
        help="Root of the directory tree for user_data execution."
    )
    parser.add_argument(
        'input_dir',
        help="Directory containing the source images."
    )
    parser.add_argument(
        'output_dir',
        help="Directory to save the output of model execution."
    )
    parser.add_argument(
        '--model_params',
        default=None,
        help="Model params (if any) combined into a single string"
    )
    return parser

def main():
    sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)
    arg_parser = make_parser()
    args = arg_parser.parse_args()
    service.user_data(args.root_dir,
                      args.input_dir,
                      args.output_dir,
                      args.model_params)



import os
import sys
import urllib
from subprocess import call
import time
from hadmlservices import callbacks

AWS_REGION = "us-east-1"

def update(cb, key, message):
    print(message)
    cb.update_job(key, message)

def user_data(root_dir,
              input_s3,
              output_s3,
              model_params):
    
    cb = callbacks.CB();
    update(cb, "job_state", "creating input and output dirs")
    for x in [root_dir + "/input", root_dir + '/output']:
        if not os.path.exists(x):
            os.mkdir(x, 0777)
        os.chmod(x, 0777)    
    
    update(cb, "job_state", "downloading log agent")
    agent = root_dir + '/awslogs-agent-setup.py'
    url = ("https://s3.amazonaws.com/aws-cloudwatch/" +
           "downloads/latest/awslogs-agent-setup.py")
    f = urllib.URLopener()
    f.retrieve(url, agent)

    # work around for ubuntu 16.04
    update(cb, "job_state", "installing awslogs.service")
    path = os.path.dirname (os.path.abspath (__file__))
    call(["cp", path + "/awslogs.service", "/lib/systemd/system/awslogs.service"])
    os.chmod("/lib/systemd/system/awslogs.service", 0644)

    update(cb, "job_state", "launching log agent")
    os.chmod(agent, 0700)
    path = os.path.dirname (os.path.abspath (__file__))
    call([agent, "-n", "-r", AWS_REGION, "-c", path + "/awslogs.conf"])

    update(cb, "started", time.strftime('%Y-%m-%d %H:%M:%S'))    

    update(cb, "job_state", "copying input data: only errors are shown")
    call(["aws", "s3", "sync", "--only-show-errors", input_s3, root_dir + "/input"])
    
    update(cb, "job_state", "executing run_script")
    if model_params:
        call([root_dir + "/run_script",
              root_dir + "/input",
              root_dir + "/output",
              model_params])
    else:
        call([root_dir + "/run_script",
              root_dir + "/input",
              root_dir + "/output"])

    update(cb, "job_state", "copying output data: only errors are shown")
    call(["aws", "s3", "sync", "--only-show-errors", "--acl", "bucket-owner-full-control",
           root_dir + "/output", output_s3])

    update(cb, "job_state", "finished")
    update(cb, "finished", time.strftime('%Y-%m-%d %H:%M:%S'))

    print("asynchronously terminating instance")
    cb.terminate_instance()
    return



"""
Provides methods to communicate with HAD ML Services (HMLS).
"""
import boto3
import json
import urllib
import botocore
import os
import time


class CB(object):

    def __init__(self):
        # TODO handle cases: 
        # 1. runs on not aws instance
        # 2. runs outside of Service environment
        self.lbd = boto3.client('lambda', region_name='us-east-1')
        #self.ec2 = boto3.client('ec2', region_name='us-east-1')
        url = "http://169.254.169.254/latest/meta-data/instance-id"
        self.instance_id = urllib.urlopen(url).read()

    def update_job(self, key, value):
        #try:
        json_params = {
            'instance_id': self.instance_id,
             key: value
        }
        lambda_fun = "had-rds-update"
        if os.environ.get('HADMLSERVICE_PROD')== '1':
            lambda_fun +=':prod'
        for i in range (0,5):
            try:
	        self.lbd.invoke(
                        FunctionName=lambda_fun,
                        InvocationType='Event',
                        Payload=json.dumps(json_params)
                )
	    except botocore.exceptions.EndpointConnectionError as e:
                print "Attempt {} out of 5 to connect to Lambda service".format(i)
                print e
                time.sleep(3)
                continue
            break
               
        """
        except botocore.exceptions.NoCredentialsError as e:
            print("NoCredentialsError: {}".format(e))
            print("Most likely running outside of HAD API Service environment.")
            return
        except botocore.exceptions.ClientError as e:
            print "ClientError: {}".format(e)
            raise
        """

    def terminate_instance(self):
        json_params = {
            'instance_id': self.instance_id
        }
        lambda_fun = "had-ec2-terminate"
        if os.environ.get('HADMLSERVICE_PROD')== '1':
            lambda_fun +=':prod'
        for i in range (0,5):
            try:
                self.lbd.invoke(
                        FunctionName=lambda_fun,
                        InvocationType='Event',
                        Payload=json.dumps(json_params)
                )
            except botocore.exceptions.EndpointConnectionError as e:
                print "Attempt {} out of 5 to connect to Lambda service".format(i)
                print e
                time.sleep(3)
                continue
            break

